<svg class='great-ds-chat__input-icon' width="26" height="22" viewBox="0 0 26 22" fill="none" xmlns="http://www.w3.org/2000/svg">
    <path id="Vector" d="M0 0V8.46165L10.458 11L0 13.5383V22L26 11L0 0Z" fill="white"/>
    </svg>
